import os
import sys
from datetime import timedelta
from django.conf import settings

DEBUG = settings.DEBUG
SERVE_STATIC = DEBUG
TEMPLATE_DEBUG = DEBUG

USE_I18N = True

USE_L10N = True

USE_TZ = True

PROTOCOL = 'http' if DEBUG else 'https'
PORT = getattr(settings, 'PORT', '8000') if DEBUG else None

if PORT and str(PORT) not in ['80', '443']:
    PORT_STRING = ':%s' % PORT
else:
    PORT_STRING = ''

SUBDIR = getattr(settings, 'SUBDIR', '')

if SUBDIR and not SUBDIR.startswith('/'):
    SUBDIR = '/%s' % SUBDIR
elif SUBDIR and SUBDIR.endswith('/'):
    SUBDIR = SUBDIR[0:-1]

SITE_URL = '%s://%s%s%s' % (PROTOCOL, settings.DOMAIN, PORT_STRING, SUBDIR)
SITE_NAME = getattr(settings, 'SITE_NAME', 'Nodeshot')

MEDIA_ROOT = '%s/media/' % settings.SITE_ROOT

MEDIA_URL = '%s/media/' % SITE_URL

STATIC_ROOT = '%s/static/' % settings.SITE_ROOT

STATIC_URL = '%s/static/' % SITE_URL

ALLOWED_HOSTS = ['*']

if PROTOCOL == 'https':
    SESSION_COOKIE_SECURE = True
    CSRF_COOKIE_SECURE = True
    SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
    os.environ['HTTPS'] = 'on'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'raven.contrib.django.raven_compat.middleware.SentryResponseErrorIdMiddleware',  # sentry
    'raven.contrib.django.raven_compat.middleware.Sentry404CatchMiddleware',  # sentry
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'corsheaders.middleware.CorsMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.request",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages"
)

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.gis',
    # hstore support
    'django_hstore',
    # celery django email backend
    'djcelery_email',
    # nodeshot
    'nodeshot.core.api',
    'nodeshot.core.layers',
    'nodeshot.core.nodes',
    'nodeshot.core.cms',
    'nodeshot.core.websockets',
    'nodeshot.interop.sync',
    'nodeshot.ui.default',
    'nodeshot.community.participation',
    'nodeshot.community.notifications',
    'nodeshot.community.profiles',
    'nodeshot.community.mailing',
    'nodeshot.networking.net',
    'nodeshot.networking.links',
    'nodeshot.networking.services',
    'nodeshot.networking.hardware',
    'nodeshot.networking.connectors',
    'nodeshot.interop.open311',
    'nodeshot.ui.open311_demo',
    # admin site
    'grappelli.dashboard',
    'grappelli',
    'filebrowser',
    'django.contrib.admin',
    # 3d parthy django apps
    'rest_framework',
    'rest_framework_swagger',
    'leaflet',
    'south',
    'smuggler',
    'reversion',
    'corsheaders',
    'social_auth',
    'rosetta',
    # other utilities
    'django_extensions',
    'debug_toolbar',
]

AUTH_USER_MODEL = 'profiles.Profile'

if getattr(settings, 'RAVEN_CONFIG', {}):
    INSTALLED_APPS.append('raven.contrib.django.raven_compat')

if 'old_nodeshot' in settings.DATABASES:
    INSTALLED_APPS.append('nodeshot.interop.oldimporter')
    DATABASE_ROUTERS = [
        'nodeshot.interop.oldimporter.db.DefaultRouter',
        'nodeshot.interop.oldimporter.db.OldNodeshotRouter'
    ]

# ------ FILEBROWSER ------ #

FILEBROWSER_DIRECTORY = ''

# ------ DJANGO CACHE ------ #

CACHES = {
    'rosetta': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'rosetta'
    }
}

if DEBUG:
    CACHES['default'] = {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
else:
    CACHES['default'] = {
        'BACKEND': 'django_redis.cache.RedisCache',
        'LOCATION': '127.0.0.1:6379:1',
        'OPTIONS': {
            'CLIENT_CLASS': 'django_redis.client.DefaultClient',
        }
    }

    SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
    SESSION_CACHE_ALIAS = 'default'

# ------ EMAIL SETTINGS ------ #

EMAIL_PORT = 1025 if DEBUG else 25  # 1025 if you are in development mode, while 25 is usually the production port

# ------ LOGGING ------ #

# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
    },
    'formatters': {
        'verbose': {
            'format': '\n\n[%(levelname)s %(asctime)s] module: %(module)s, process: %(process)d, thread: %(thread)d\n%(message)s'
        },
    },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'logfile': {
            'level': 'ERROR',
            'class': 'logging.handlers.RotatingFileHandler',
            'formatter': 'verbose',
            'filename': settings.SITE_ROOT + "/../log/nodeshot.error.log",
            'maxBytes': 10485760,  # 10 MB
            'backupCount': 3,
            'formatter': 'verbose'
        },
    },
    'loggers': {
        'django': {
            'handlers': ['logfile'],
            'level': 'ERROR',
            'propagate': True,
        },
        'django.request': {
            'handlers': ['mail_admins', 'logfile'],
            'level': 'ERROR',
            'propagate': True,
        },
        '': {
            'handlers': ['logfile'],
            'level': 'ERROR',
        },
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        }
    }
}

# ------ CELERY ------ #

if DEBUG:
    # this app makes it possible to use django as a queue system for celery
    # so you don't need to install RabbitQM or Redis
    # pretty cool for development, but might not suffice for production if your system is heavily used
    # our suggestion is to switch only if you start experiencing issues
    INSTALLED_APPS.append('kombu.transport.django')
    BROKER_URL = 'django://'
    # synchronous behaviour for development
    # more info here: http://docs.celeryproject.org/en/latest/configuration.html#celery-always-eager
    CELERY_ALWAYS_EAGER = True
    CELERY_EAGER_PROPAGATES_EXCEPTIONS = True
else:
    # in production the default background queue manager is Redis
    BROKER_URL = 'redis://localhost:6379/0'
    CELERY_RESULT_BACKEND = 'redis://localhost:6379/0'
    BROKER_TRANSPORT_OPTIONS = {
        "visibility_timeout": 3600,  # 1 hour
        "fanout_prefix": True
    }
    # in production emails are sent in the background
    EMAIL_BACKEND = 'djcelery_email.backends.CeleryEmailBackend'

CELERYBEAT_SCHEDULE = {
    'purge_notifications': {
        'task': 'nodeshot.community.notifications.tasks.purge_notifications',
        'schedule': timedelta(days=1),
    }
}

# ------ GRAPPELLI ------ #

if 'grappelli' in INSTALLED_APPS:
    GRAPPELLI_ADMIN_TITLE = '<a href="%s">%s Admin</a>' % (SITE_URL, SITE_NAME)
    GRAPPELLI_INDEX_DASHBOARD = 'nodeshot.dashboard.NodeshotDashboard'

# ------ DEBUG TOOLBAR ------ #

INTERNAL_IPS = ('127.0.0.1', '::1',)  # ip addresses where you want to show the debug toolbar here
DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
}

# ------ UNIT TESTING SPEED UP ------ #

SOUTH_DATABASE_ADAPTERS = {'default': 'south.db.postgresql_psycopg2'}
SOUTH_TESTS_MIGRATE = False

if 'test' in sys.argv:
    CELERY_ALWAYS_EAGER = True

    PASSWORD_HASHERS = (
        'django.contrib.auth.hashers.MD5PasswordHasher',
        'django.contrib.auth.hashers.SHA1PasswordHasher',
        'django.contrib.auth.hashers.PBKDF2PasswordHasher',
        'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
        'django.contrib.auth.hashers.BCryptPasswordHasher',
    )

# ------ CORS-HEADERS SETTINGS ------ #

CORS_ORIGIN_ALLOW_ALL = True

# ------ SOCIAL AUTH SETTINGS ------ #

if 'social_auth' in INSTALLED_APPS:
    MIDDLEWARE_CLASSES += ('social_auth.middleware.SocialAuthExceptionMiddleware',)

    # In Django 1.6, the default session serliazer has been switched to one based on JSON,
    # rather than pickles, to improve security. Django-openid-auth does not support this
    # because it attemps to store content that is not JSON serializable in sessions.
    # See https://docs.djangoproject.com/en/dev/releases/1.6/#default-session-serialization-switched-to-json
    # for details on the Django 1.6 change.
    SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'

    AUTHENTICATION_BACKENDS = (
        'django.contrib.auth.backends.ModelBackend',
        'nodeshot.community.profiles.backends.EmailBackend',
        'social_auth.backends.facebook.FacebookBackend',
        'social_auth.backends.google.GoogleOAuth2Backend',
        'nodeshot.community.profiles.social_auth_extra.github.GithubBackend',
    )

    SOCIAL_AUTH_PIPELINE = (
        'social_auth.backends.pipeline.social.social_auth_user',
        'social_auth.backends.pipeline.associate.associate_by_email',
        'social_auth.backends.pipeline.user.get_username',
        'nodeshot.community.profiles.social_auth_extra.pipeline.create_user',
        'social_auth.backends.pipeline.social.associate_user',
        'nodeshot.community.profiles.social_auth_extra.pipeline.load_extra_data',
        'social_auth.backends.pipeline.user.update_user_details'
    )

    SOCIAL_AUTH_ENABLED_BACKENDS = ('facebook', 'google', 'github')

    # register a new app:
    FACEBOOK_APP_ID = ''  # put your app id
    FACEBOOK_API_SECRET = ''
    FACEBOOK_EXTENDED_PERMISSIONS = ['email', 'user_about_me', 'user_birthday', 'user_hometown']

    GOOGLE_OAUTH2_CLIENT_ID = ''
    GOOGLE_OAUTH2_CLIENT_SECRET = ''

    # register a new app:
    GITHUB_APP_ID = ''
    GITHUB_API_SECRET = ''
    GITHUB_EXTENDED_PERMISSIONS = ['user:email']

    SOCIAL_AUTH_DEFAULT_USERNAME = 'new_social_auth_user'
    SOCIAL_AUTH_UUID_LENGTH = 3
    SOCIAL_AUTH_SESSION_EXPIRATION = False
    SOCIAL_AUTH_ASSOCIATE_BY_MAIL = True

    LOGIN_URL = '/'
    LOGIN_REDIRECT_URL = '/'
    LOGIN_ERROR_URL = '/'

# ------ DJANGO-LEAFLET SETTINGS ------ #

LEAFLET_CONFIG = {
    'DEFAULT_CENTER': (49.06775, 30.62011),
    'DEFAULT_ZOOM': 4,
    'MIN_ZOOM': 1,
    'MAX_ZOOM': 18,
    'TILES': [
        ('Map', 'http://otile1.mqcdn.com/tiles/1.0.0/map/{z}/{x}/{y}.png', '&copy; <a href="http://www.openstreetmap.org/copyright" target="_blank">OpenStreetMap</a> contributors | Tiles Courtesy of <a href="http://www.mapquest.com/" target="_blank">MapQuest</a>'),
        ('Satellite', 'http://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}', 'Source: <a href="http://www.esri.com/">Esri</a> &copy; and the GIS User Community ')
    ],
    'ATTRIBUTION_PREFIX': '<a href="http://github.com/ninuxorg/nodeshot" target="_blank">Nodeshot</a>',
    'RESET_VIEW': False,
}

# ------ DJANGO-ROSETTA SETTINGS ------ #

ROSETTA_CACHE_NAME = 'rosetta'
ROSETTA_MESSAGES_PER_PAGE = 50
ROSETTA_EXCLUDED_APPLICATIONS = [app for app in INSTALLED_APPS if not app.startswith('nodeshot')]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Metric.query'
        db.add_column(u'metrics_metric', 'query',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True),
                      keep_default=False)

        # Adding unique constraint on 'Metric', fields ['object_id', 'name', 'content_type', 'tags']
        db.create_unique(u'metrics_metric', ['object_id', 'name', 'content_type_id', 'tags'])


    def backwards(self, orm):
        # Removing unique constraint on 'Metric', fields ['object_id', 'name', 'content_type', 'tags']
        db.delete_unique(u'metrics_metric', ['object_id', 'name', 'content_type_id', 'tags'])

        # Deleting field 'Metric.query'
        db.delete_column(u'metrics_metric', 'query')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'metrics.metric': {
            'Meta': {'unique_together': "(('name', 'tags', 'content_type', 'object_id'),)", 'object_name': 'Metric'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 3, 23, 0, 0)'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '75'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'query': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'tags': ('jsonfield.fields.JSONField', [], {'default': '{}', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 3, 23, 0, 0)'})
        }
    }

    complete_apps = ['metrics']


from django.contrib import admin

from nodeshot.core.base.admin import BaseGeoAdmin
from .models import Link


class LinkAdmin(BaseGeoAdmin):
    list_display = (
        '__unicode__',
        'type',
        'status',
        'interface_a_mac',
        'interface_b_mac',
        'added',
        'updated'
    )
    list_filter = ('status', 'type')
    date_hierarchy = 'added'
    ordering = ('-id',)

    raw_id_fields = ('interface_a', 'interface_b', 'node_a', 'node_b')
    autocomplete_lookup_fields = {
        'fk': ['interface_a', 'interface_b', 'node_a', 'node_b'],
    }

    readonly_fields = [
        'first_seen', 'last_seen',
        'metric_value', 'min_rate', 'max_rate',
        'dbm', 'noise'
    ] + BaseGeoAdmin.readonly_fields[:]
    exclude = ('shortcuts',)


admin.site.register(Link, LinkAdmin)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from . import Profile


class SocialLink(BaseDate):
    """
    External links like website or social network profiles
    """
    user = models.ForeignKey(Profile, verbose_name=_('user'))
    url = models.URLField(_('url'))
    description = models.CharField(_('description'), max_length=128, blank=True)

    class Meta:  # NOQA
        ordering = ['id']
        app_label = 'profiles'
        db_table = 'profiles_social_links'
        unique_together = ('user', 'url')

    def __unicode__(self):
        return self.url



from .settings import ACCESS_LEVELS as BASE_ACCESS_LEVELS


TIME_ZONES = (
    ('GMT-12', '(GMT-12:00) International Date Line West'),
    ('GMT-11', '(GMT-11:00) Midway Island, Samoa'),
    ('GMT-10', '(GMT-10:00) Hawaii'),
    ('GMT-9', '(GMT-9:00) Alaska'),
    ('GMT-8', '(GMT-8:00) Pacific Standard Time'),
    ('GMT-7', '(GMT-7:00) Mountain Standard Time'),
    ('GMT-6', '(GMT-6:00) Central Standard Time'),
    ('GMT-5', '(GMT-5:00) Eastern Standard Time'),
    ('GMT-4:30', '(GMT-4:30) Caracas'),
    ('GMT-4', '(GMT-4:00) Eastern Caribbean Time'),
    ('GMT-3:30', '(GMT-3:30) Newfoundland'),
    ('GMT-3', '(GMT-3:00) Greenland, Buenos Aires'),
    ('GMT-2', '(GMT-2:00) Mid-Atlantic'),
    ('GMT-1', '(GMT-1:00) Cape Verde Time'),
    ('GMT', '(GMT) Western Europe Time'),
    ('GMT+1', '(GMT+1:00) Central European Time'),
    ('GMT+2', '(GMT+2:00) Eastern European Time'),
    ('GMT+3', '(GMT+3:00) Baghdad, Riyadh, Moscow, St. Petersburg'),
    ('GMT+3:30', '(GMT+3:30) Tehran'),
    ('GMT+4:00', '(GMT+4:00) Abu Dhabi, Muscat, Baku, Tbilisi'),
    ('GMT+4:30', '(GMT+4:30) Kabul'),
    ('GMT+5', '(GMT+5:00) Ekaterinburg, Islamabad, Karachi, Tashkent'),
    ('GMT+5:30', '(GMT+5:30) Bombay, Calcutta, Madras, New Delhi'),
    ('GMT+5:45', '(GMT+5:45) Kathmandu'),
    ('GMT+6:00', '(GMT+6:00) Almaty, Dhaka, Colombo'),
    ('GMT+7:', '(GMT+7:00) Bangkok, Hanoi, Jakarta'),
    ('GMT+8', '(GMT+8:00) Beijing, Perth, Singapore, Hong Kong'),
    ('GMT+9', '(GMT+9:00) Tokyo, Seoul, Osaka, Sapporo, Yakutsk'),
    ('GMT+9:30', '(GMT+9:30) Adelaide, Darwin'),
    ('GMT+10', '(GMT+10:00) Eastern Australia, Guam, Vladivostok'),
    ('GMT+11', '(GMT+11:00) Magadan, Solomon Islands, New Caledonia'),
    ('GMT+12', '(GMT+12:00) Auckland, Wellington, Fiji, Kamchatka'),
)

ACCESS_LEVELS = BASE_ACCESS_LEVELS.copy()
ACCESS_LEVELS['public'] = 0

PLANNED_STATUS = (
    (0, 'pending'),
    (1, 'completed'),
    (-1, 'cancelled')
)



from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q

from rest_framework import authentication, generics

from nodeshot.core.base.mixins import ACLMixin, CustomDataMixin
from nodeshot.core.nodes.models import Node

from .permissions import IsOwnerOrReadOnly
from .serializers import *
from .models import *


# ------ DEVICES ------ #


class DeviceList(ACLMixin, generics.ListAPIView):
    """
    Retrieve device list according to user access level
    
    Parameters:
    
     * `search=<word>`: search <word> in name, slug, description and address of nodes
     * `limit=<n>`: specify number of items per page (defaults to 40)
     * `limit=0`: turns off pagination
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Device.objects.all().select_related('node')
    serializer_class = DeviceListSerializer
    pagination_serializer_class = PaginatedDeviceSerializer
    paginate_by_param = 'limit'
    paginate_by = 40
    
    def get_queryset(self):
        """
        Optionally restricts the returned devices
        by filtering against a `search` query parameter in the URL.
        """
        # retrieve all devices which are published and accessible to current user
        # and use joins to retrieve related fields
        queryset = super(DeviceList, self).get_queryset()#.select_related('layer', 'status', 'user')
        
        # retrieve value of querystring parameter "search"
        search = self.request.QUERY_PARAMS.get('search', None)
        
        if search is not None:
            search_query = (
                Q(name__icontains=search) |
                Q(description__icontains=search)
            )
            # add instructions for search to queryset
            queryset = queryset.filter(search_query)
        
        return queryset
    
device_list = DeviceList.as_view()


class DeviceDetails(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
    """
    Retrieve details of specified device.
    
    ### DELETE
    
    Delete specified device. Must be authenticated as owner or admin.
    
    ### PUT & PATCH
    
    Edit device. Must be authenticated as owner or admin.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)
    queryset = Device.objects.all().select_related('node')
    serializer_class = DeviceDetailSerializer

device_details = DeviceDetails.as_view()


class NodeDeviceList(CustomDataMixin, generics.ListCreateAPIView):
    """
    Retrieve devices of specified node according to user access level.
    
    Parameters:
    
     * `limit=<n>`: specify number of items per page (defaults to 40)
     * `limit=0`: turns off pagination
    
    ### POST
    
    Create a new device for this node. Must be authenticated as node owner or admin.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)
    serializer_class = NodeDeviceListSerializer
    serializer_custom_class = DeviceAddSerializer
    pagination_serializer_class = PaginatedNodeDeviceSerializer
    paginate_by_param = 'limit'
    paginate_by = 40
    
    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'node': self.node.id
        }
    
    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only devices of current node
        """
        super(NodeDeviceList, self).initial(request, *args, **kwargs)
        
        # ensure node exists
        try:
            self.node = Node.objects.published()\
                        .accessible_to(request.user)\
                        .get(slug=self.kwargs.get('slug', None))
        except Node.DoesNotExist:
            raise Http404(_('Node not found.'))
        
        # check permissions on node (for device creation)
        self.check_object_permissions(request, self.node)
        
        # return only devices of current node
        self.queryset = Device.objects.filter(node_id=self.node.id)\
                        .accessible_to(self.request.user)\
                        .select_related('node')
    
node_device_list = NodeDeviceList.as_view()


# ------ INTERFACES ------ #


class BaseInterfaceList(CustomDataMixin, generics.ListCreateAPIView):
    """
    Retrieve interface list for specified device.
    
    ### POST
    
    Create new interface for specified device.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)
    
    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'device': self.device.id
        }
    
    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure device exists and store it in an instance attribute
            * change queryset to return only devices of current node
        """
        super(BaseInterfaceList, self).initial(request, *args, **kwargs)
        
        # ensure device exists
        try:
            self.device = Device.objects.accessible_to(request.user)\
                        .get(pk=self.kwargs.get('pk', None))
        except Device.DoesNotExist:
            raise Http404(_('Device not found.'))
        
        # check permissions on device (for interface creation)
        self.check_object_permissions(request, self.device)
        
        # return only interfaces of current device
        self.queryset = self.model.objects.filter(device_id=self.device.id)\
                        .accessible_to(self.request.user)


class BaseInterfaceDetails(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
    """
    Retrieve details of specified interface.
    
    ### DELETE
    
    Delete specified interface. Must be authenticated as owner or admin.
    
    ### PUT & PATCH
    
    Edit interface. Must be authenticated as owner or admin.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)


class DeviceEthernetList(BaseInterfaceList):
    model = Ethernet
    serializer_class = EthernetDetailSerializer
    serializer_custom_class = EthernetAddSerializer
    
device_ethernet_list = DeviceEthernetList.as_view()


class EthernetDetails(BaseInterfaceDetails):
    queryset = Ethernet.objects.all()
    serializer_class = EthernetSerializer

ethernet_details = EthernetDetails.as_view()


class DeviceWirelessList(BaseInterfaceList):
    model = Wireless
    serializer_class = WirelessDetailSerializer
    serializer_custom_class = WirelessAddSerializer
    
device_wireless_list = DeviceWirelessList.as_view()


class WirelessDetails(BaseInterfaceDetails):
    queryset = Wireless.objects.all()
    serializer_class = WirelessSerializer

wireless_details = WirelessDetails.as_view()


class DeviceBridgeList(BaseInterfaceList):
    model = Bridge
    serializer_class = BridgeDetailSerializer
    serializer_custom_class = BridgeAddSerializer
    
device_bridge_list = DeviceBridgeList.as_view()


class BridgeDetails(BaseInterfaceDetails):
    queryset = Bridge.objects.all()
    serializer_class = BridgeSerializer

bridge_details = BridgeDetails.as_view()


class DeviceTunnelList(BaseInterfaceList):
    model = Tunnel
    serializer_class = TunnelDetailSerializer
    serializer_custom_class = TunnelAddSerializer
    
device_tunnel_list = DeviceTunnelList.as_view()


class TunnelDetails(BaseInterfaceDetails):
    queryset = Tunnel.objects.all()
    serializer_class = TunnelSerializer

tunnel_details = TunnelDetails.as_view()


class DeviceVlanList(BaseInterfaceList):
    model = Vlan
    serializer_class = VlanDetailSerializer
    serializer_custom_class = VlanAddSerializer
    
device_vlan_list = DeviceVlanList.as_view()


class VlanDetails(BaseInterfaceDetails):
    queryset = Vlan.objects.all()
    serializer_class = VlanSerializer

vlan_details = VlanDetails.as_view()


# ------ IP ADDRESS ------ #


class InterfaceIpList(CustomDataMixin, generics.ListCreateAPIView):
    """
    Retrieve IP list of specified interface.
    
    ### POST
    
    Create new interface for specified device.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)
    model = Ip
    serializer_class = IpDetailSerializer
    serializer_custom_class = IpAddSerializer
    
    def get_custom_data(self):
        """ additional request.DATA """
        return {
            'interface': self.interface.id
        }
    
    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure interface exists and store it in an instance attribute
            * change queryset to return only devices of current node
        """
        super(InterfaceIpList, self).initial(request, *args, **kwargs)
        
        # ensure interface exists
        try:
            self.interface = Interface.objects.accessible_to(request.user)\
                        .get(pk=self.kwargs.get('pk', None))
        except Interface.DoesNotExist:
            raise Http404(_('Interface not found.'))
        
        # check permissions on interface (for interface creation)
        self.check_object_permissions(request, self.interface)
        
        # return only interfaces of current interface
        self.queryset = self.model.objects.filter(interface_id=self.interface.id)\
                        .accessible_to(self.request.user)

interface_ip_list = InterfaceIpList.as_view()


class IpDetails(ACLMixin, generics.RetrieveUpdateDestroyAPIView):
    """
    Retrieve details of specified ip address.
    
    ### DELETE
    
    Delete specified ip address. Must be authenticated as owner or admin.
    
    ### PUT & PATCH
    
    Edit ip address. Must be authenticated as owner or admin.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (IsOwnerOrReadOnly,)
    queryset = Ip.objects.all()
    serializer_class = IpDetailSerializer

ip_details = IpDetails.as_view()



import simplejson as json

from django.test.client import Client
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError
from django.core import mail, management
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.base.tests import user_fixtures, BaseTestCase
from nodeshot.core.base.utils import ago
from nodeshot.core.nodes.models import Node

from .models import *
from .settings import settings, DEFAULT_DISTANCE, DEFAULT_BOOLEAN, REGISTER
from .tasks import purge_notifications

# TODO: cleanup this mess
# remove websockets from installed apps and disconnect signals
if 'nodeshot.core.websockets' in settings.INSTALLED_APPS:
    from importlib import import_module
    from nodeshot.core.websockets import settings as websocket_settings

    for module in websocket_settings.REGISTER:
        module = import_module(module)
        module.disconnect()

    setattr(websocket_settings, 'REGISTER', [])

    settings.INSTALLED_APPS = [app for app in settings.INSTALLED_APPS if app != 'nodeshot.core.websockets']


class TestNotification(BaseTestCase):
    """
    Test Notifications
    """

    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
    ]

    def setUp(self):
        # empty outbox, emails generated from EmailAddress model in nodeshot.community.profiles
        mail.outbox = []

    def test_notification_to_herself(self):
        """ An user cannot send a notification to herself/himself """
        n = Notification(
            to_user_id=1,
            from_user_id=1,
            type='custom',
            text='test'
        )
        self.assertRaises(ValidationError, n.full_clean)

    def test_email_is_sent(self):
        """ Test custom notification is sent via email """
        n = Notification(
            to_user_id=1,
            from_user_id=2,
            type='custom',
            text='testing test'
        )
        n.save()

        self.assertEqual(Notification.objects.count(), 1)
        self.assertEqual(len(mail.outbox), 1)

        email_text = n.email_message
        self.assertIn('Hi', email_text)
        self.assertIn('testing test', email_text)
        self.assertIn('More details here', email_text)
        self.assertIn('This is an automatic notification sent from from', email_text)

    def test_purge_notifications(self):
        n = Notification.objects.create(**{
            "to_user_id": 1,
            "type": "custom",
            "text": "testing"
        })
        # not enough old to be purged
        n.added = ago(days=3)
        n.save(auto_update=False)
        management.call_command('purge_notifications')
        # no notifications have been deleted
        self.assertEqual(Notification.objects.count(), 1)

        # enough old to be purged
        n.added = ago(days=41)
        n.save(auto_update=False)
        management.call_command('purge_notifications')
        self.assertEqual(Notification.objects.count(), 0)

        n = Notification.objects.create(**{
            "to_user_id": 1,
            "type": "custom",
            "text": "testing"
        })
        n.added = ago(days=41)
        n.save(auto_update=False)
        purge_notifications.delay()
        self.assertEqual(Notification.objects.count(), 0)

    if 'nodeshot.community.notifications.registrars.nodes' in REGISTER:
        def test_check_settings(self):
            n = Notification(**{
                "to_user_id": 4,
                "type": "node_own_status_changed",
            })
            self.assertEqual(n.check_user_settings(medium='web'), DEFAULT_BOOLEAN)
            self.assertTrue(n.check_user_settings(medium='email'), DEFAULT_DISTANCE)

        def test_node_created_to_all(self):
            # set every user to receive notifications about any node created
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_created = 0
                user.email_notification_settings.save()
                user.web_notification_settings.node_created = 0
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })

            # ensure all users except node owner have been notified about the node creation
            self.assertEqual(Notification.objects.count(), all_users.count()-1)
            self.assertEqual(len(mail.outbox), all_users.count()-1)

            # ensure owner notification object for owner has not been created in DB
            self.assertEqual(Notification.objects.filter(to_user_id=1).count(), 0)

        def test_node_created_to_all_web_noone_mail(self):
            all_users = User.objects.all()

            for user in all_users:
                # no one wants to receive emails
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.save()
                # everyone wants to receive web notifications about all nodes
                user.web_notification_settings.node_created = 0
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })

            # ensure all users except node owner have been notified about the node creation
            self.assertEqual(Notification.objects.count(), all_users.count()-1)
            self.assertEqual(len(mail.outbox), 0)

            # ensure owner notification object for owner has not been created in DB
            self.assertEqual(Notification.objects.filter(to_user_id=1).count(), 0)

        def test_node_created_to_all_email_noone_web(self):
            all_users = User.objects.all()

            for user in all_users:
                # no one wants to receive emails
                user.email_notification_settings.node_created = 0
                user.email_notification_settings.save()
                # everyone wants to receive web notifications about all nodes
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })

            # ensure all users except node owner have been notified about the node creation
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), all_users.count()-1)

        def test_node_created_to_noone(self):
            # set every user to NOT receive notifications about new nodes
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })

            # ensure no notifications are created in the DB nor sent
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            # web notifications enabled
            for user in all_users:
                user.web_notification_settings.node_created = 0
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification 2',
                'slug': 'test-notification 2',
                'layer_id': 1,
                'geometry': 'POINT (-2.16 47.12)',
                'user_id': 1
            })

            # ensure notifications are created
            self.assertEqual(Notification.objects.count(), all_users.count()-1)
            # but emails not sent
            self.assertEqual(len(mail.outbox), 0)

        def test_node_created_distance(self):
            # set every user to NOT receive notifications about new nodes
            # disable email notifications for all users
            for user in User.objects.all():
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.save()

            # creat node of user who will receive the notification
            fusolab = Node.objects.create(**{
                'name': 'a node in rome',
                'slug': 'a-node-in-rome',
                'layer_id': 1,
                'geometry': 'POINT (12.5822391919000012 41.8720419276999820)',
                'user_id': 4  # romano
            })
            # ensure no notifications have been created/sent
            self.assertEqual(len(mail.outbox), 0)
            self.assertEqual(Notification.objects.count(), 0)

            # enable notification in distance range only for romano
            user = User.objects.get(username='romano')
            user.email_notification_settings.node_created = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_created = 20
            user.web_notification_settings.save()

            # romano should receive a notification when this new near node is created
            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })

            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 1)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 1)
            # ensure notification is directed towards him
            notification = Notification.objects.all().order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 4)

            # a new node is created at a distance wich is too far for romano's preferences
            far_node = Node.objects.create(**{
                'name': 'too far',
                'slug': 'too far',
                'layer_id': 1,
                'geometry': 'POINT (13.100 41.401)',
                'user_id': 1
            })

            # ensure no notifications are created nor email sent
            self.assertEqual(Notification.objects.count(), 1)
            self.assertEqual(len(mail.outbox), 1)

            # RESET
            near_node.delete()
            far_node.delete()
            Notification.objects.all().delete()

            # romano now wants to be notified via email but doesn't want to see the web notification
            user.email_notification_settings.node_created = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_created = -1
            user.web_notification_settings.save()

            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })
            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 2)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 0)

        def test_node_deleted_all(self):
            # set every user to receive notifications about any node deletion
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_deleted = 0
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = 0
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            node.delete()

            self.assertEqual(Notification.objects.count(), all_users.count()-1)
            self.assertEqual(len(mail.outbox), all_users.count()-1)

        def test_node_deleted_all_email_noone_web(self):
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_deleted = 0
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            node.delete()

            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), all_users.count()-1)

        def test_node_deleted_all_web_noone_email(self):
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = 0
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            node.delete()

            self.assertEqual(Notification.objects.count(), all_users.count()-1)
            self.assertEqual(len(mail.outbox), 0)

        def test_node_deleted_distance(self):
            # set every user to NOT receive notifications about new nodes
            # disable email notifications for all users
            for user in User.objects.all():
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.save()
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.save()

            # create node of user who will receive the notification
            fusolab = Node.objects.create(**{
                'name': 'a node in rome',
                'slug': 'a-node-in-rome',
                'layer_id': 1,
                'geometry': 'POINT (12.5822391919000012 41.8720419276999820)',
                'user_id': 4  # romano
            })

            # enable notification in distance range only for romano
            user = User.objects.get(username='romano')
            user.email_notification_settings.node_deleted = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_deleted = 20
            user.web_notification_settings.save()

            # romano should receive a notification when this new near node is created
            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })

            # ensure 0 notifications
            self.assertEqual(len(mail.outbox), 0)
            self.assertEqual(Notification.objects.count(), 0)

            # delete node
            near_node.delete()

            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 1)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 1)
            # ensure notification is directed towards him
            notification = Notification.objects.all().order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 4)

            # a new node is created at a distance wich is too far for romano's preferences
            far_node = Node.objects.create(**{
                'name': 'too far',
                'slug': 'too far',
                'layer_id': 1,
                'geometry': 'POINT (13.100 41.401)',
                'user_id': 1
            })

            # ensure 0 notifications
            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(Notification.objects.count(), 1)

            far_node.delete()

            # ensure no additional notifications are created nor email sent
            self.assertEqual(Notification.objects.count(), 1)
            self.assertEqual(len(mail.outbox), 1)

            # RESET
            Notification.objects.all().delete()

            # romano now wants to be notified via email but doesn't want to see the web notification
            user.email_notification_settings.node_deleted = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_deleted = -1
            user.web_notification_settings.save()

            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })
            near_node.delete()
            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 2)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 0)

            # RESET
            Notification.objects.all().delete()

            # yes to web no to email
            user.email_notification_settings.node_deleted = -1
            user.email_notification_settings.save()
            user.web_notification_settings.node_deleted = 20
            user.web_notification_settings.save()

            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })
            near_node.delete()
            # no additional mail sent
            self.assertEqual(len(mail.outbox), 2)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 1)
            # ensure notification is directed towards him
            notification = Notification.objects.all().order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 4)

        def test_node_status_changed_all(self):
            all_users = User.objects.all()

            # all users receive notifications about status changes but no other notification
            for user in all_users:
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.node_status_changed = 0
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.node_status_changed = 0
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            node.status_id = 3
            node.save()

            self.assertEqual(Notification.objects.filter(type='node_status_changed').count(), all_users.count()-1)
            self.assertEqual(Notification.objects.filter(type='node_own_status_changed').count(), 1)
            # ensure notification of type "node_own_status_changed" is directed towards owner
            notification = Notification.objects.filter(type='node_own_status_changed').order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 1)
            self.assertEqual(len(mail.outbox), 8)  # all users have received emails

        def test_node_status_changed_all_email_noone_web(self):
            all_users = User.objects.all()

            for user in all_users:
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.node_status_changed = 0
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.node_status_changed = -1
                user.web_notification_settings.save()

            node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (-2.46 48.12)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            node.status_id = 3
            node.save()

            self.assertEqual(Notification.objects.filter(type='node_status_changed').count(), 0)
            self.assertEqual(Notification.objects.filter(type='node_own_status_changed').count(), 1)
            # ensure notification of type "node_own_status_changed" is directed towards owner
            notification = Notification.objects.filter(type='node_own_status_changed').order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 1)
            self.assertEqual(len(mail.outbox), 8)  # all users have received emails

        def test_node_status_changed_distance(self):
            all_users = User.objects.all()

            # disable everything
            for user in all_users:
                user.email_notification_settings.node_deleted = -1
                user.email_notification_settings.node_created = -1
                user.email_notification_settings.node_status_changed = -1
                user.email_notification_settings.node_own_status_changed = False
                user.email_notification_settings.save()
                user.web_notification_settings.node_deleted = -1
                user.web_notification_settings.node_created = -1
                user.web_notification_settings.node_status_changed = -1
                user.web_notification_settings.node_own_status_changed = False
                user.web_notification_settings.save()

            # create node of user who will receive the notification
            fusolab = Node.objects.create(**{
                'name': 'a node in rome',
                'slug': 'a-node-in-rome',
                'layer_id': 1,
                'geometry': 'POINT (12.5822391919000012 41.8720419276999820)',
                'user_id': 4  # romano
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            # enable notification in distance range only for romano
            user = User.objects.get(username='romano')
            user.email_notification_settings.node_status_changed = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_status_changed = 20
            user.web_notification_settings.save()

            # romano should receive a notification when this new near node is created
            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })
            # ensure 0 notifications
            self.assertEqual(Notification.objects.count(), 0)
            self.assertEqual(len(mail.outbox), 0)

            near_node.status_id = 3
            near_node.save()

            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 1)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 1)
            self.assertEqual(Notification.objects.filter(type='node_status_changed').count(), 1)
            # ensure notification is directed towards romano
            notification = Notification.objects.all().order_by('-id')[0]
            self.assertEqual(notification.to_user_id, 4)

            # a new node is created at a distance wich is too far for romano's preferences
            far_node = Node.objects.create(**{
                'name': 'too far',
                'slug': 'too far',
                'layer_id': 1,
                'geometry': 'POINT (13.100 41.401)',
                'user_id': 1
            })

            far_node.status_id = 3
            far_node.save()

            # ensure no additional notifications are created nor email sent
            self.assertEqual(Notification.objects.count(), 1)
            self.assertEqual(len(mail.outbox), 1)

            # RESET
            near_node.delete()
            far_node.delete()
            Notification.objects.all().delete()

            # romano now wants to be notified via email but doesn't want to see the web notification
            user.email_notification_settings.node_status_changed = 20  # 20 km
            user.email_notification_settings.save()
            user.web_notification_settings.node_status_changed = -1
            user.web_notification_settings.save()

            near_node = Node.objects.create(**{
                'name': 'test notification',
                'slug': 'test-notification',
                'layer_id': 1,
                'geometry': 'POINT (12.5454 41.8352)',
                'user_id': 1
            })

            near_node.status_id = 3
            near_node.save()

            # ensure 1 email is sent
            self.assertEqual(len(mail.outbox), 2)
            # ensure 1 notification is created
            self.assertEqual(Notification.objects.count(), 0)

        def test_read_notifications_API(self):
            """ test read notification API operation """
            url = reverse('api_notification_list')

            # GET 403 unauthorized
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403)

            self.client.login(username='romano', password='tester')

            response = self.client.get(url)
            self.assertContains(response, '[]')

            user = User.objects.get(pk=4)
            user.web_notification_settings.node_created = 0
            user.web_notification_settings.save()

            # generate a notification
            fusolab = Node.objects.create(**{
                'name': 'a node in rome',
                'slug': 'a-node-in-rome',
                'layer_id': 1,
                'geometry': 'POINT (12.5822391919000012 41.8720419276999820)',
                'user_id': 1  # admin
            })

            # test ?action=count
            response = self.client.get(url, { 'action': 'count' })
            self.assertContains(response, '{"count": 1}')

            # test ?read=false
            # retrieve notifications but do not mark as read
            response = self.client.get(url, { 'read': 'false' })
            self.assertEquals(1, len(response.data), 'expected 1 notification')

            # test ?read=true
            # repeating the request should still show 1 unread notification, but the read operation will mark them as read
            self.assertEqual(Notification.objects.filter(to_user_id=4, is_read=False).count(), 1)
            response = self.client.get(url)
            self.assertEquals(1, len(response.data))
            # repeating again should find 0 unread notifications
            response = self.client.get(url)
            self.assertContains(response, '[]')
            self.assertEqual(Notification.objects.filter(to_user_id=4, is_read=False).count(), 0)

            # test action=all
            response = self.client.get(url, { 'action': 'all' })
            notifications = response.data
            self.assertEquals(1, len(notifications['results']))
            self.assertContains(response, '"is_read"', msg_prefix="'read' should be a field on its own")

            # test pagination of action=all
            for i in range(1, 31):
                notification_text = 'testing notifications, iteration n. %d' % i
                n = Notification(
                    to_user_id=4,
                    type='custom',
                    text=notification_text
                )
                n.save()
            response = self.client.get(url, { 'action': 'all' })
            notifications = response.data
            self.assertEquals(31, notifications['count'])
            self.assertTrue(notifications['next'] is not None, 'expected a next page')
            self.assertTrue(notifications['previous'] is None, 'expected no previous page')
            response = self.client.get(notifications['next'])
            notifications = response.data
            self.assertTrue(notifications['previous'] is not None, 'expected a previous page')

            # test non expected action should default to unread
            response = self.client.get(url, { 'action': 'doesntexist' })
            self.assertEquals(30, len(response.data))

        def test_notification_detail_API(self):
            # set user #4 to receive notifications
            user = User.objects.get(pk=4)
            user.web_notification_settings.node_created = 0
            user.web_notification_settings.save()

            # generate a notification
            fusolab = Node.objects.create(**{
                'name': 'a node in rome',
                'slug': 'a-node-in-rome',
                'layer_id': 1,
                'geometry': 'POINT (12.5822391919000012 41.8720419276999820)',
                'user_id': 1  # admin
            })

            notification_id = Notification.objects.order_by('-id').all()[0].id

            # test notification detail
            url = reverse('api_notification_detail', args=[notification_id])

            # GET 403 unauthorized
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403)

            # GET 404
            # login as wrong user
            self.client.login(username='registered', password='tester')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 404)
            self.client.logout()

            # GET 200
            # login as right user
            self.client.login(username='romano', password='tester')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.data['id'], notification_id)

        def test_email_notification_settings(self):
            """ ensure email notification settings endpoint works correctly """
            url = reverse('api_notification_email_settings')

            # GET 401: unauthenticated request should fail
            c = Client()
            response = c.get(url)
            self.assertContains(response, 'credentials', status_code=403)

            self.client.login(username='romano', password='tester')

            # GET 200
            response = self.client.get(url)
            self.assertEquals(200, response.status_code)
            self.assertContains(response, 'uri')
            self.assertContains(response, 'node_created')
            self.assertContains(response, 'node_status_changed')
            self.assertContains(response, 'node_own_status_changed')
            self.assertContains(response, 'node_deleted')

            # PATCH 200
            response = self.client.patch(url, { "node_own_status_changed": False })
            self.assertEquals(200, response.status_code)
            # check DB
            user = User.objects.get(username='romano')
            self.assertFalse(user.email_notification_settings.node_own_status_changed)

            # PUT 200
            response = self.client.put(url, json.dumps({
                "node_created": 50,
                "node_status_changed": 50,
                "node_own_status_changed": True,
                "node_deleted": 50
            }), content_type='application/json')
            self.assertEquals(200, response.status_code)
            # check DB
            user = User.objects.get(username='romano')
            self.assertTrue(user.email_notification_settings.node_own_status_changed)
            self.assertEqual(user.email_notification_settings.node_created, 50)
            self.assertEqual(user.email_notification_settings.node_status_changed, 50)
            self.assertEqual(user.email_notification_settings.node_deleted, 50)

        def test_web_notification_settings(self):
            """ ensure web notification settings endpoint works correctly """
            url = reverse('api_notification_web_settings')

            # GET 401: unauthenticated request should fail
            c = Client()
            response = c.get(url)
            self.assertContains(response, 'credentials', status_code=403)

            self.client.login(username='romano', password='tester')

            # GET 200
            response = self.client.get(url)
            self.assertEquals(200, response.status_code)
            self.assertContains(response, 'uri')
            self.assertContains(response, 'node_created')
            self.assertContains(response, 'node_status_changed')
            self.assertContains(response, 'node_own_status_changed')
            self.assertContains(response, 'node_deleted')

            # PATCH 200
            response = self.client.patch(url, { "node_own_status_changed": False })
            self.assertEquals(200, response.status_code)
            # check DB
            user = User.objects.get(username='romano')
            self.assertFalse(user.web_notification_settings.node_own_status_changed)

            # PUT 200
            response = self.client.put(url, json.dumps({
                "node_created": 50,
                "node_status_changed": 50,
                "node_own_status_changed": True,
                "node_deleted": 50
            }), content_type='application/json')
            self.assertEquals(200, response.status_code)
            # check DB
            user = User.objects.get(username='romano')
            self.assertTrue(user.web_notification_settings.node_own_status_changed)
            self.assertEqual(user.web_notification_settings.node_created, 50)
            self.assertEqual(user.web_notification_settings.node_status_changed, 50)
            self.assertEqual(user.web_notification_settings.node_deleted, 50)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Metric'
        db.create_table(u'metrics_metric', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2015, 3, 5, 0, 0))),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2015, 3, 5, 0, 0))),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=75)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('tags', self.gf('jsonfield.fields.JSONField')(default={}, blank=True)),
        ))
        db.send_create_signal(u'metrics', ['Metric'])

    def backwards(self, orm):
        # Deleting model 'Metric'
        db.delete_table(u'metrics_metric')

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'metrics.metric': {
            'Meta': {'object_name': 'Metric'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 3, 5, 0, 0)'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '75'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'tags': ('jsonfield.fields.JSONField', [], {'default': '{}', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 3, 5, 0, 0)'})
        }
    }

    complete_apps = ['metrics']



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies='nodeshot.core.nodes',
    module='nodeshot.networking.net'
)


from routing_protocol import RoutingProtocol
from device import Device
from interface import Interface
from ip import Ip

from interfaces.ethernet import Ethernet
from interfaces.wireless import Wireless
from interfaces.bridge import Bridge
from interfaces.tunnel import Tunnel
from interfaces.vlan import Vlan
from interfaces.vap import Vap


__all__ = [
    'RoutingProtocol',
    'Device',
    'Interface',
    'Ip',
    'Ethernet',
    'Wireless',
    'Bridge',
    'Tunnel',
    'Vlan',
    'Vap'
]


# ------ Add relationship to ExtensibleNodeSerializer ------ #

from nodeshot.core.nodes.base import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(**{
    'name': 'devices',
    'view_name': 'api_node_devices',
    'lookup_field': 'slug'
})


from datetime import datetime

from django.contrib.gis.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _

from jsonfield import JSONField

from nodeshot.core.base.models import BaseDate

from .utils import query, write


class Metric(BaseDate):
    name = models.CharField(_('name'), max_length=75)
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    related_object = generic.GenericForeignKey('content_type', 'object_id')
    tags = JSONField(_('tags'), blank=True, default={})
    query = models.CharField(_('query'), blank=True, max_length=255, help_text='default query')

    class Meta:
        unique_together = ('name', 'tags', 'content_type', 'object_id')

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.related_object:
            self.tags.update({
                'content_type': self.content_type.name,
                'object_id': str(self.object_id)
            })
        if not self.query:
            self.query = self.select(sql_only=True)
        super(Metric, self).save(*args, **kwargs)

    def write(self, values, timestamp=None, database=None):
        """ write metric point """
        return write(name=self.name,
                     values=values,
                     tags=self.tags,
                     timestamp=timestamp,
                     database=database)

    def select(self, fields=[], since=None, limit=None, q=None, sql_only=False):
        if q is not None and ('DROP' in q or 'DELETE' in q):
            q = None
        if not q:
            if fields:
                fields = ', '.join(fields)
            else:
                fields = '*'
            if not since:
                since = 'now() - 30d'
            if isinstance(since, datetime):
                since = "'{0}'".format(since.strftime('%Y-%m-%dT%H:%M:%SZ'))
            conditions = "time >= {0}".format(since)
            tags = ' AND '.join(["{0} = '{1}'".format(*tag) for tag in self.tags.items()])
            if tags:
                conditions = '{0} AND {1}'.format(conditions, tags)
            q = 'SELECT {fields} FROM {name} WHERE {conditions}'.format(fields=fields,
                                                                        name=self.name,
                                                                        conditions=conditions)
            if limit:
                q = '{0} LIMIT {1}'.format(q, limit)
        if sql_only:
            return q
        # return query
        return query(q)


# from django.contrib.auth.models import User
from django.contrib.auth import get_user_model
from django.contrib.auth.signals import user_logged_in
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
User = get_user_model()


@receiver(user_logged_in, dispatch_uid='user_loggedin')
def user_loggedin(sender, **kwargs):
    """ collect metrics about user logins """
    tags = {
        'user_id': str(kwargs['user'].pk),
        'username': kwargs['user'].username,
    }
    values = {
        'value': 1,
        'path': kwargs['request'].path
    }
    write('user_logins', values=values, tags=tags)


@receiver(post_delete, sender=User, dispatch_uid='user_created')
def user_created(sender, **kwargs):
    """ collect metrics about users unsubscribing """
    write('user_variations', {'variation': -1}, tags={'action': 'deleted'})
    write('user_count', {'total': User.objects.count()})


@receiver(post_save, sender=User, dispatch_uid='user_deleted')
def user_deleted(sender, **kwargs):
    """ collect metrics about new users signing up """
    if kwargs.get('created'):
        write('user_variations', {'variation': 1}, tags={'action': 'created'})
        write('user_count', {'total': User.objects.count()})



from django.conf import settings


NETENGINE_BACKENDS = getattr(settings, 'NODESHOT_NETENGINE_BACKENDS', [
    ('netengine.backends.ssh.AirOS', 'AirOS (SSH)'),
    ('netengine.backends.ssh.OpenWRT', 'OpenWRT (SSH)'),
    ('netengine.backends.snmp.AirOS', 'AirOS (SNMP)'),
])



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.networking.links.views',  # noqa
    url(r'^links/$', 'link_list', name='api_link_list'),
    url(r'^links/(?P<pk>[0-9]+)/$', 'link_details', name='api_link_details'),
    # geojson
    url(r'^links.geojson$', 'link_geojson_list', name='api_links_geojson_list'),
    url(r'^links/(?P<pk>[0-9]+).geojson$', 'link_geojson_details', name='api_links_geojson_details'),
    # node links
    url(r'^nodes/(?P<slug>[-\w]+)/links/$', 'node_link_list', name='api_node_links'),
)



from django.conf.urls import patterns, include, url
from django.conf import settings
from django.contrib import admin

from filebrowser.sites import site

from nodeshot.community.profiles.settings import EMAIL_CONFIRMATION

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls))
)

if 'smuggler' in settings.INSTALLED_APPS:
    # smuggler must be before admin
    urlpatterns = patterns('',
        url(r'^admin/', include('smuggler.urls'))
    ) + urlpatterns

if 'filebrowser' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^admin/filebrowser/', include(site.urls)),
    )

if 'rosetta' in settings.INSTALLED_APPS:
    # rosetta must be before admin
    urlpatterns = patterns('',
        url(r'^admin/translations/', include('rosetta.urls')),
    ) + urlpatterns

if 'nodeshot.interop.sync' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'', include('nodeshot.interop.sync.urls')),
    )

if settings.DEBUG and settings.SERVE_STATIC:
    urlpatterns += patterns('django.contrib.staticfiles.views',
        url(r'^static/(?P<path>.*)$', 'serve'),
        url(r'^media/(?P<path>.*)$', 'serve'),
    )

if 'social_auth' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'', include('social_auth.urls')),
    )

if 'grappelli' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^grappelli/', include('grappelli.urls')),
    )

if 'nodeshot.core.websockets' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^websockets/', include('nodeshot.core.websockets.urls')),
    )

if 'nodeshot.community.profiles' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^account/password/reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<key>.+)/$',
            'nodeshot.community.profiles.html_views.password_reset_from_key',
            name='account_password_reset_from_key'),
    )

if 'nodeshot.community.profiles' in settings.INSTALLED_APPS and EMAIL_CONFIRMATION:
    urlpatterns += patterns('',
        url(r'^confirm_email/(\w+)/$',
            'nodeshot.community.profiles.html_views.confirm_email',
            name='emailconfirmation_confirm_email'),
    )

# include 'nodeshot.core.api.urls'
if 'nodeshot.core.api' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'', include('nodeshot.core.api.urls')),
    )

if 'nodeshot.ui.open311_demo' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^open311/$', include('nodeshot.ui.open311_demo.urls', namespace='open311_demo', app_name='open311_demo')),
    )

if 'nodeshot.ui.default' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'', include('nodeshot.ui.default.urls', namespace='ui', app_name='ui')),
    )

urlpatterns += patterns('',
    url(r'^jsi18n/$', 'nodeshot.core.base.views.jsi18n', {'packages': ('nodeshot.ui.default',)}, name='jsi18n')
)



from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.db.models import Q
from django.utils.module_loading import import_by_path

from nodeshot.core.layers.models import Layer

from optparse import make_option


class Command(BaseCommand):
    args = '<layer_slug layer_slug ...>'
    help = 'Synchronize external layers with the local database'

    option_list = BaseCommand.option_list + (
        make_option(
            '--exclude',
            action='store',
            dest='exclude',
            default=[],
            help='Exclude specific layers from synchronization\n\
                 Supply a comma separated string of layer slugs\n\
                 e.g. --exclude=layer1-slug,layer2-slug,layer3-slug\n\
                 (works only if no layer has been specified)'
        ),
    )

    def retrieve_layers(self, *args, **options):
        """
        Retrieve specified layers or all external layers if no layer specified.
        """

        # init empty Q object
        queryset = Q()

        # if no layer specified
        if len(args) < 1:
            # cache queryset
            all_layers = Layer.objects.published().external()

            # check if there is any layer to exclude
            if options['exclude']:
                # convert comma separated string in python list, ignore spaces
                exclude_list = options['exclude'].replace(' ', '').split(',')
                # retrieve all layers except the ones specified in exclude list
                return all_layers.exclude(slug__in=exclude_list)
            else:
                # nothing to exclude, retrieve all layers
                self.verbose('no layer specified, will retrieve all layers!')
                return all_layers

        # otherwise loop over args and retrieve each specified layer
        for layer_slug in args:
            queryset = queryset | Q(slug=layer_slug)

            # verify existence
            try:
                # retrieve layer
                layer = Layer.objects.get(slug=layer_slug)

                # raise exception if layer is not external
                if not layer.is_external:
                    raise CommandError('Layer "%s" is not an external layer\n\r' % layer_slug)

                # raise exception if layer is not published
                if not layer.is_published:
                    raise CommandError('Layer "%s" is not published. Why are you trying to work on an unpublished layer?\n\r' % layer_slug)

            # raise exception if one of the layer looked for doesn't exist
            except Layer.DoesNotExist:
                raise CommandError('Layer "%s" does not exist\n\r' % layer_slug)

        # return published external layers
        return Layer.objects.published().external().select_related().filter(queryset)

    def verbose(self, message):
        if self.verbosity == 2:
            self.stdout.write('%s\n\r' % message)

    def handle(self, *args, **options):
        """ execute sync command """
        # store verbosity level in instance attribute for later use
        self.verbosity = int(options.get('verbosity'))

        # blank line
        self.stdout.write('\r\n')

        # retrieve layers
        layers = self.retrieve_layers(*args, **options)

        if len(layers) < 1:
            self.stdout.write('no layers to process\n\r')
            return
        else:
            self.verbose('going to process %d layers...' % len(layers))

        # loop over
        for layer in layers:
            # retrieve interop class if available
            try:
                synchronizer_path = layer.external.synchronizer_path
            except (ObjectDoesNotExist, AttributeError):
                self.stdout.write('External Layer %s does not have a synchronizer class specified\n\r' % layer.name)
                continue

            # if no synchronizer_path jump to next layer
            if synchronizer_path == 'None':
                self.stdout.write('External Layer %s does not have a synchronizer class specified\n\r' % layer.name)
                continue

            if layer.external.config is None:
                self.stdout.write('Layer %s does not have a config yet\n\r' % layer.name)
                continue

            # retrieve class
            Synchronizer = import_by_path(synchronizer_path)
            self.stdout.write('imported module %s\r\n' % Synchronizer.__name__)

            # try running
            try:
                instance = Synchronizer(layer, verbosity=self.verbosity)
                self.stdout.write('Processing layer "%s"\r\n' % layer.slug)
                messages = instance.process()
            except ImproperlyConfigured, e:
                self.stdout.write('Validation error: %s\r\n' % e)
                continue

            for message in messages:
                self.stdout.write('%s\n\r' % message)

        self.stdout.write('\r\n')



import simplejson as json

from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse

from nodeshot.core.base.tests import BaseTestCase
from nodeshot.core.base.tests import user_fixtures


class ServiceTest(BaseTestCase):
    
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_routing_protocols.json',
        'test_devices.json',
        'test_interfaces.json',
        'test_ip_addresses.json',
        'test_services.json'
    ]
    
    def test_api_service_category_list(self):
        # GET: 200 - service category list
        url = reverse('api_service_category_list')
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        
    def test_api_service_category_details(self):
        # GET: 200 - service category detail
        url = reverse('api_service_category_details', args=[1])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
    
    def test_api_service_list(self):
        # GET: 200 - service list
        url = reverse('api_service_list')
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        
    def test_api_service_details(self):
        # GET: 200 - service detail
        url = reverse('api_service_details', args=[1])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)



from rest_framework import generics, authentication

from nodeshot.core.base.mixins import ACLMixin

# cache
from rest_framework_extensions.cache.decorators import cache_response
from nodeshot.core.base.cache import cache_by_group

from .serializers import *
from .models import *


class PageList(ACLMixin, generics.ListAPIView):
    """
    Retrieve the list of all the available pages.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Page.objects.published()
    serializer_class = PageListSerializer

    @cache_response(86400, key_func=cache_by_group)
    def get(self, request, *args, **kwargs):
        return super(PageList, self).get(request, *args, **kwargs)

page_list = PageList.as_view()


class PageDetail(ACLMixin, generics.RetrieveAPIView):
    """
    Retrieve specified page.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Page.objects.published()
    serializer_class = PageDetailSerializer

    @cache_response(86400, key_func=cache_by_group)
    def get(self, request, *args, **kwargs):
        return super(PageDetail, self).get(request, *args, **kwargs)

page_detail = PageDetail.as_view()


# ------ Menu ------ #


class MenuList(ACLMixin, generics.ListAPIView):
    """
    Retrieve menu item list.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = MenuItem.objects.published().filter(parent=None)
    serializer_class = MenuSerializer

    @cache_response(86400, key_func=cache_by_group)
    def get(self, request, *args, **kwargs):
        return super(MenuList, self).get(request, *args, **kwargs)

menu_list = MenuList.as_view()



import requests
import simplejson as json
from datetime import datetime

from social_auth.models import UserSocialAuth

from ..settings import EMAIL_CONFIRMATION


def create_user(backend, details, response, uid, username, user=None, *args,
                **kwargs):
    """Create user. Depends on get_username pipeline."""
    if user:
        return {'user': user}
    if not username:
        return None

    # Avoid hitting field max length
    email = details.get('email')
    original_email = None
    if email and UserSocialAuth.email_max_length() < len(email):
        original_email = email
        email = ''

    return {
        'user': UserSocialAuth.create_user(username=username, email=email,
                                           sync_emailaddress=False),
        'original_email': original_email,
        'is_new': True
    }


def load_extra_data(backend, details, response, uid, user, social_user=None,
                    *args, **kwargs):
    """Load extra data from provider and store it on current UserSocialAuth
    extra_data field.
    """
    social_user = social_user or \
                  UserSocialAuth.get_social_auth(backend.name, uid)

    if kwargs['is_new'] and EMAIL_CONFIRMATION:
        from ..models import EmailAddress
        emailaddress = EmailAddress(**{
            'user': user,
            'email': user.email,
            'verified': True,
            'primary': True
        })
        emailaddress.save()

    if social_user:
        extra_data = backend.extra_data(user, uid, response, details)
        if kwargs.get('original_email') and 'email' not in extra_data:
            extra_data['email'] = kwargs.get('original_email')
        if extra_data and social_user.extra_data != extra_data:
            if social_user.extra_data:
                social_user.extra_data.update(extra_data)
            else:
                social_user.extra_data = extra_data
            social_user.save()

        if backend.name == 'facebook' and kwargs['is_new']:
            response = json.loads(requests.get('https://graph.facebook.com/%s?access_token=%s' % (extra_data['id'], extra_data['access_token'])).content)

            try:
                user.city, user.country = response.get('hometown').get('name').split(', ')
            except AttributeError:
                pass

            try:
                user.birth_date = datetime.strptime(response.get('birthday'), '%m/%d/%Y').date()
            except AttributeError:
                pass

            user.save()

        return {'social_user': social_user}



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'StatusIcon', fields ['status', 'layer']
        db.delete_unique('nodes_status_icon', ['status_id', 'layer_id'])

        # Deleting model 'StatusIcon'
        db.delete_table('nodes_status_icon')

        # Adding field 'Status.stroke_width'
        db.add_column('nodes_status', 'stroke_width',
                      self.gf('django.db.models.fields.SmallIntegerField')(default=0),
                      keep_default=False)

        # Adding field 'Status.fill_color'
        db.add_column('nodes_status', 'fill_color',
                      self.gf('nodeshot.core.base.fields.RGBColorField')(default='', max_length=7, blank=True),
                      keep_default=False)

        # Adding field 'Status.stroke_color'
        db.add_column('nodes_status', 'stroke_color',
                      self.gf('nodeshot.core.base.fields.RGBColorField')(default='#000000', max_length=7, blank=True),
                      keep_default=False)

        # Adding field 'Status.text_color'
        db.add_column('nodes_status', 'text_color',
                      self.gf('nodeshot.core.base.fields.RGBColorField')(default='#FFFFFF', max_length=7, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Adding model 'StatusIcon'
        db.create_table('nodes_status_icon', (
            ('status', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['nodes.Status'])),
            ('layer', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['layers.Layer'], null=True, blank=True)),
            ('marker', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('icon', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('background_color', self.gf('nodeshot.core.base.fields.RGBColorField')(max_length=7, blank=True)),
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('foreground_color', self.gf('nodeshot.core.base.fields.RGBColorField')(max_length=7, blank=True)),
        ))
        db.send_create_signal('nodes', ['StatusIcon'])

        # Adding unique constraint on 'StatusIcon', fields ['status', 'layer']
        db.create_unique('nodes_status_icon', ['status_id', 'layer_id'])

        # Deleting field 'Status.stroke_width'
        db.delete_column('nodes_status', 'stroke_width')

        # Deleting field 'Status.fill_color'
        db.delete_column('nodes_status', 'fill_color')

        # Deleting field 'Status.stroke_color'
        db.delete_column('nodes_status', 'stroke_color')

        # Deleting field 'Status.text_color'
        db.delete_column('nodes_status', 'text_color')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'layers.layer': {
            'Meta': {'object_name': 'Layer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'area': ('django.contrib.gis.db.models.fields.PolygonField', [], {'null': 'True', 'blank': 'True'}),
            'center': ('django.contrib.gis.db.models.fields.PointField', [], {'null': 'True', 'blank': 'True'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_external': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mantainers': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['profiles.Profile']", 'symmetrical': 'False', 'blank': 'True'}),
            'minimum_distance': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'new_nodes_allowed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'organization': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'zoom': ('django.db.models.fields.SmallIntegerField', [], {'default': '12'})
        },
        'nodes.image': {
            'Meta': {'ordering': "['order']", 'object_name': 'Image'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'node': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['nodes.Node']"}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'})
        },
        'nodes.node': {
            'Meta': {'object_name': 'Node'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'elev': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'geometry': ('django.contrib.gis.db.models.fields.GeometryField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'layer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['layers.Layer']"}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '75'}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75', 'blank': 'True'}),
            'status': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['nodes.Status']", 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['profiles.Profile']", 'null': 'True', 'blank': 'True'})
        },
        'nodes.status': {
            'Meta': {'ordering': "['order']", 'object_name': 'Status'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'fill_color': ('nodeshot.core.base.fields.RGBColorField', [], {'max_length': '7', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75'}),
            'stroke_color': ('nodeshot.core.base.fields.RGBColorField', [], {'default': "'#000000'", 'max_length': '7', 'blank': 'True'}),
            'stroke_width': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'text_color': ('nodeshot.core.base.fields.RGBColorField', [], {'default': "'#FFFFFF'", 'max_length': '7', 'blank': 'True'})
        },
        'profiles.profile': {
            'Meta': {'object_name': 'Profile'},
            'about': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '150', 'blank': 'True'}),
            'birth_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 2, 24, 0, 0)'}),
            'email': ('django.db.models.fields.EmailField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254', 'db_index': 'True'})
        }
    }

    complete_apps = ['nodes']


from django.conf import settings


DEFAULT_SYNCHRONIZERS = [
    ('nodeshot.interop.sync.synchronizers.Nodeshot', 'Nodeshot (RESTful translator)'),
    ('nodeshot.interop.sync.synchronizers.GeoJson', 'GeoJSON (periodic sync)'),
    ('nodeshot.interop.sync.synchronizers.GeoRss', 'GeoRSS (periodic sync)'),
    ('nodeshot.interop.sync.synchronizers.OpenWisp', 'OpenWisp (periodic sync)')
]

SYNCHRONIZERS = DEFAULT_SYNCHRONIZERS + getattr(settings, 'NODESHOT_SYNCHRONIZERS', [])



from __future__ import absolute_import
from .django import DefaultUiDjangoTest  # noqa
from .selenium import DefaultUiSeleniumTest

__all__ = [
    'DefaultUiDjangoTest',
    'DefaultUiSeleniumTest'
]



from __future__ import absolute_import

from django.core.urlresolvers import reverse
from django.test import TestCase

from nodeshot.core.base.tests import user_fixtures
from nodeshot.ui.default import settings as local_settings


class DefaultUiDjangoTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_images.json',
    ]

    def test_index(self):
        response = self.client.get(reverse('ui:index'))
        self.assertEqual(response.status_code, 200)

    def test_social_auth_optional(self):
        # enable social auth
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', True)
        response = self.client.get(reverse('ui:index'))
        self.assertContains(response, 'social-buttons')
        # disable social auth
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', False)
        response = self.client.get(reverse('ui:index'))
        self.assertNotContains(response, 'social-buttons')

    def test_facebook_optional(self):
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', True)
        setattr(local_settings, 'FACEBOOK_ENABLED', True)
        response = self.client.get(reverse('ui:index'))
        self.assertContains(response, 'btn-facebook')
        setattr(local_settings, 'FACEBOOK_ENABLED', False)
        response = self.client.get(reverse('ui:index'))
        self.assertNotContains(response, 'btn-facebook')
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', False)

    def test_google_optional(self):
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', True)
        setattr(local_settings, 'GOOGLE_ENABLED', True)
        response = self.client.get(reverse('ui:index'))
        self.assertContains(response, 'btn-google')
        setattr(local_settings, 'GOOGLE_ENABLED', False)
        response = self.client.get(reverse('ui:index'))
        self.assertNotContains(response, 'btn-google')
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', False)

    def test_github_optional(self):
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', True)
        setattr(local_settings, 'GITHUB_ENABLED', True)
        response = self.client.get(reverse('ui:index'))
        self.assertContains(response, 'btn-github')
        setattr(local_settings, 'GITHUB_ENABLED', False)
        response = self.client.get(reverse('ui:index'))
        self.assertNotContains(response, 'btn-github')
        setattr(local_settings, 'SOCIAL_AUTH_ENABLED', False)



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies='nodeshot.core.nodes',
    module='nodeshot.community.mailing'
)


from inward import Inward
from outward import Outward

__all__ = [
    'Inward',
    'Outward',
]


# ------ Add relationship to ExtensibleNodeSerializer ------ #

from nodeshot.core.nodes.serializers import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(**{
    'name': 'contact',
    'view_name': 'api_node_contact',
    'lookup_field': 'slug'
})


import json
from time import sleep

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.base.tests import user_fixtures
from nodeshot.core.base.utils import ago

from . import settings as local_settings
TEST_DATABASE = '{0}_test'.format(local_settings.INFLUXDB_DATABASE)
setattr(local_settings, 'INFLUXDB_DATABASE', TEST_DATABASE)

from .models import Metric
from .utils import get_db, query, create_database


class MetricsTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
    ]

    @classmethod
    def setUpClass(cls):
        create_database()

    @classmethod
    def tearDownClass(cls):
        client = get_db()
        client.drop_database(TEST_DATABASE)

    def test_metric_model(self):
        metric = Metric(name='test_metric')
        metric.full_clean()
        metric.save()
        self.assertEqual(str(metric), metric.name)

    def test_metric_tags_from_related_object(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        self.assertItemsEqual(metric.tags, {
            'content_type': metric.content_type.name,
            'object_id': metric.object_id
        })

    def test_get_db(self):
        self.assertIn('<influxdb.client.InfluxDBClient object at', str(get_db()))

    def test_query(self):
        databases = query('show databases')['databases']
        self.assertEqual(type(databases), list)
        databases = [database['name'] for database in databases]
        self.assertIn(TEST_DATABASE, databases)

    def test_write(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        metric.write({'value1': 1, 'value2': 'string'})
        sleep(1)
        series = query('show series')
        self.assertIn('test_metric', series.keys())
        sleep(2)
        sql = "select * from test_metric where content_type = 'user' and object_id = '{0}'".format(metric.object_id)
        points = query(sql)['test_metric']
        self.assertEqual(len(points), 1)
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_write_timestamp_string(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        timestamp_string = '2015-03-06T14:18:12Z'
        metric.write({'value1': 1, 'value2': 'string'}, timestamp=timestamp_string)
        sleep(2)
        sql = "select * from test_metric where content_type = 'user' and object_id = '{0}'".format(metric.object_id)
        points = query(sql)['test_metric']
        self.assertEqual(len(points), 1)
        self.assertEqual(points[0]['time'], timestamp_string)
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_write_timestamp_datetime(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        datetime = ago(days=365)
        metric.write({'value1': 1, 'value2': 'string'}, timestamp=datetime)
        sleep(2)
        sql = "select * from test_metric where content_type = 'user' and object_id = '{0}'".format(metric.object_id)
        points = query(sql)['test_metric']
        self.assertEqual(len(points), 1)
        self.assertEqual(points[0]['time'], datetime.strftime('%Y-%m-%dT%H:%M:%SZ'))
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_select(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        metric.write({'value1': 1})
        sleep(0.5)
        metric.write({'value1': 2})
        sleep(0.5)
        metric.write({'value1': 3})
        sleep(2)
        self.assertEqual(len(metric.select()['test_metric']), 3)
        self.assertEqual(len(metric.select(limit=1)['test_metric']), 1)
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_signal(self):
        self.assertNotEqual(query('show series'), {})

    def test_metric_details_get(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        metric.write({'value1': 1, 'value2': 'string'})
        sleep(1)
        response = self.client.get('/api/v1/metrics/{0}/'.format(metric.pk))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, metric.select())
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_metric_details_post(self):
        metric = Metric(name='test_metric')
        metric.related_object = User.objects.first()
        metric.full_clean()
        metric.save()
        metric.write({'value': 1})
        url = '/api/v1/metrics/{0}/'.format(metric.pk)
        sleep(1)
        # post 400
        response = self.client.post(url)
        self.assertEqual(response.status_code, 400)
        # post application/json
        response = self.client.post(url, json.dumps({'value': 2}),
                                    content_type='application/json')
        self.assertEqual(response.status_code, 200)
        self.assertItemsEqual(response.data, {'detail': 'ok'})
        sleep(1)
        self.assertEqual(metric.select()['test_metric'][-1]['value'], 2)
        # post application/x-www-form-urlencoded
        response = self.client.post(url, 'value=3',
                                    content_type='application/x-www-form-urlencoded')
        self.assertEqual(response.status_code, 200)
        self.assertItemsEqual(response.data, {'detail': 'ok'})
        sleep(1)
        self.assertEqual(metric.select()['test_metric'][-1]['value'], 3)
        # post multipart/form-data
        response = self.client.post(url, {'value': 4})
        self.assertEqual(response.status_code, 200)
        self.assertItemsEqual(response.data, {'detail': 'ok'})
        sleep(1)
        self.assertEqual(metric.select()['test_metric'][-1]['value'], 4)
        # post multipart/form-data (float)
        response = self.client.post(url, {'value': 5.0})
        self.assertEqual(response.status_code, 200)
        self.assertItemsEqual(response.data, {'detail': 'ok'})
        sleep(1)
        self.assertEqual(metric.select()['test_metric'][-1]['value'], 5.0)
        # drop series
        series_id = query('show series')['test_metric'][0]['_id']
        query('drop measurement test_metric')
        query('drop series {0}'.format(series_id))

    def test_add_admin(self):
        self.client.login(username='admin', password='tester')
        response = self.client.get(reverse('admin:metrics_metric_add'))
        self.assertEqual(response.status_code, 200)



from rest_framework import serializers
from rest_framework.pagination import PaginationSerializer

from nodeshot.core.base.serializers import HyperlinkedField

from .models import *


__all__ = [
    'UnreadNotificationSerializer',
    'NotificationSerializer',
    'PaginatedNotificationSerializer',
    'EmailNotificationSettingsSerializer',
    'WebNotificationSettingsSerializer'
]


class UnreadNotificationSerializer(serializers.ModelSerializer):
    """
    Unread notification serializer
    """
    from_user_id = serializers.Field(source='from_user_id')
    from_user_detail = serializers.HyperlinkedRelatedField(
        source='from_user',
        view_name='api_profile_detail',
        read_only=True
    )
    related_object = serializers.SerializerMethodField('get_related_object')
    
    def get_related_object(self, obj):
        related = obj.related_object
        if related is not None:
            if hasattr(related, 'slug'):
                return related.slug
            else:
                return related.id
        return None
    
    class Meta:
        model = Notification
        fields = (
            'id', 'type', 'is_read', 'from_user_id',
            'from_user_detail', 'related_object',
            'text', 'added'
        )


class NotificationSerializer(UnreadNotificationSerializer):
    """
    Notification serializer
    """
    class Meta:
        model = Notification
        fields = (
            'id', 'type', 'is_read', 'from_user_id',
            'from_user_detail', 'related_object',
            'text', 'added'
        )


class PaginatedNotificationSerializer(PaginationSerializer):
    """
    Serializes page objects of notification querysets.
    """
    class Meta:
        object_serializer_class = NotificationSerializer


class EmailNotificationSettingsSerializer(serializers.ModelSerializer):
    """
    Email Notification Settings serializer
    """
    uri = HyperlinkedField(view_name='api_notification_email_settings')
    
    class Meta:
        model = UserEmailNotificationSettings
        exclude = ('id', 'user',)


class WebNotificationSettingsSerializer(serializers.ModelSerializer):
    """
    Web Notification Settings serializer
    """
    uri = HyperlinkedField(view_name='api_notification_email_settings')
    
    class Meta:
        model = UserWebNotificationSettings
        exclude = ('id', 'user',)



import simplejson as json

from django.test import TestCase
from django.test.client import Client
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.http import int_to_base36
from django.core import mail
from django.forms import ValidationError

from .settings import settings, EMAIL_CONFIRMATION

if EMAIL_CONFIRMATION:
    from .models import EmailAddress, EmailConfirmation

from .models import Profile as User
from .models import PasswordReset, SocialLink


class ProfilesTest(TestCase):
    fixtures = [
        'initial_data.json',
        'test_profiles.json',
        'test_layers.json',
        'test_status.json',
    ]

    def setUp(self):
        self.client.login(username='registered', password='tester')
        if EMAIL_CONFIRMATION:
            for user in User.objects.all():
                user.save()
            # set all email addresses as verified
            for email_address in EmailAddress.objects.all():
                email_address.verified = True
                email_address.save()
        mail.outbox = []

    def test_new_users_have_default_group(self):
        """ users should have a default group when created """
        # ensure owner of node fusolab has at least one group
        self.assertEqual(User.objects.get(username='romano').groups.count(), 1)

        # create new user and check if it has any group
        new_user = User(username='new_user_test', email='new_user@testing.com', password='tester', is_active=True)
        new_user.save()
        # retrieve again from DB just in case...
        new_user = User.objects.get(username='new_user_test')
        self.assertEqual(new_user.groups.filter(name='registered').count(), 1)

    def test_user_admin(self):
        self.client.logout()
        self.client.login(username='admin', password='tester')
        # create new user through admin
        url = reverse('admin:profiles_profile_add')
        with self.assertRaises(ValidationError):
            self.client.post(url, {
                'username': 'testing',
                'password': 'password',
                'password2': 'password'
            })

    def test_profile_list_API(self):
        """ test new user creation through the API """
        url = reverse('api_profile_list')
        un_auth_client = Client()

        # GET 401
        response = un_auth_client.get(url)
        self.assertEqual(401, response.status_code)

        # GET 200
        response = self.client.get(url)
        self.assertContains(response, 'username')
        self.assertNotContains(response, 'password_confirmation')

        # POST 400: missing required field
        self.client.logout()
        new_user = {
            "username": "new_user_test",
            "email": "new_user@testing.com",
            "password": "new_user_test"
        }
        response = self.client.post(url, json.dumps(new_user), content_type='application/json')
        self.assertContains(response, 'password_confirmation', status_code=400)

        # POST 400: Password confirmation mismatch
        new_user['password_confirmation'] = 'WRONG'
        response = self.client.post(url, json.dumps(new_user), content_type='application/json')
        self.assertContains(response, _('Password confirmation mismatch'), status_code=400)

        # POST 201: Created
        new_user['password_confirmation'] = 'new_user_test'
        response = self.client.post(url, json.dumps(new_user), content_type='application/json')
        self.assertNotContains(response, 'password_confirmation', status_code=201)

        user = User.objects.get(username='new_user_test')
        self.assertEqual(user.is_active, not EMAIL_CONFIRMATION)
        # ensure password is hashed
        self.assertTrue(user.has_usable_password())

        if EMAIL_CONFIRMATION:
            email_address = EmailAddress.objects.get(email='new_user@testing.com')
            self.assertTrue(email_address.primary)
            self.assertFalse(email_address.verified)
            self.assertEqual(email_address.emailconfirmation_set.count(), 1)

            key = email_address.emailconfirmation_set.all()[0].key
            confirmation_url = reverse('emailconfirmation_confirm_email', args=[key])
            response = self.client.get(confirmation_url)
            self.assertEqual(response.status_code, 302)
            self.assertIn('_auth_user_id', self.client.session)

            user = User.objects.get(username='new_user_test')
            self.assertEqual(user.is_active, True)

            # retrieve from DB again
            email_address = EmailAddress.objects.get(email='new_user@testing.com')
            self.assertTrue(email_address.verified)
            self.assertTrue(email_address.primary)

    def test_profile_detail_API(self):
        """ test new user creation through the API """
        url = reverse('api_profile_detail', args=['registered'])

        # GET 200
        response = self.client.get(url)
        self.assertContains(response, 'username')
        self.assertNotContains(response, 'password')
        self.assertNotContains(response, 'email')

        # PUT 200
        profile = {
            "first_name": "Registered",
            "last_name": "User",
            "about": "Lorem ipsum dolor...",
            "gender": "M",
            "birth_date": "1987-03-23",
            "address": "Via Prova 1",
            "city": "Rome",
            "country": "Italy"
        }
        response = self.client.put(url, json.dumps(profile), content_type='application/json')
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(username='registered')
        self.assertEqual(user.first_name, 'Registered')
        self.assertEqual(user.last_name, 'User')
        self.assertEqual(user.about, 'Lorem ipsum dolor...')
        self.assertEqual(user.gender, 'M')
        self.assertEqual(user.birth_date.strftime('%Y-%m-%d'), '1987-03-23')
        self.assertEqual(user.address, 'Via Prova 1')
        self.assertEqual(user.city, 'Rome')
        self.assertEqual(user.country, 'Italy')

        # PUT 403
        # try modifying another user's profile
        url = reverse('api_profile_detail', args=['romano'])
        response = self.client.put(url, json.dumps(profile), content_type='application/json')
        self.assertEqual(response.status_code, 403)

        self.client.logout()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_profile_detail_API_get_location(self):
        url = reverse('api_profile_detail', args=['registered'])
        user = User.objects.get(username='registered')

        # expects None
        user.city = ''
        user.country = ''
        user.save()
        response = self.client.get(url)
        self.assertEqual(response.data['location'], None)

        # expects City, Country
        user.city = 'Rome'
        user.country = 'Italy'
        user.save()
        response = self.client.get(url)
        self.assertEqual(response.data['location'], 'Rome, Italy')

        # expects City
        user.city = 'Rome'
        user.country = ''
        user.save()
        response = self.client.get(url)
        self.assertEqual(response.data['location'], 'Rome')

        # expects Country
        user.city = ''
        user.country = 'Italy'
        user.save()
        response = self.client.get(url)
        self.assertEqual(response.data['location'], 'Italy')

    def test_account_detail_API(self):
        url = reverse('api_account_detail')

        un_auth_client = Client()

        # GET 401
        response = un_auth_client.get(url)
        self.assertEqual(401, response.status_code)

        # GET 200
        response = self.client.get(url)
        self.assertContains(response, 'profile')
        self.assertContains(response, 'change_password')

    def test_account_password_change_API(self):
        url = reverse('api_account_password_change')

        un_auth_client = Client()

        # GET 401
        response = un_auth_client.get(url)
        self.assertEqual(401, response.status_code)

        # GET 405
        response = self.client.get(url)
        self.assertEqual(405, response.status_code)

        # POST 400: wrong current password
        new_password = {}
        response = self.client.post(url, new_password)
        self.assertContains(response, 'current_password', status_code=400)
        self.assertContains(response, 'password1', status_code=400)
        self.assertContains(response, 'password2', status_code=400)

        # POST 400: wrong current password
        new_password = {
            "current_password": "wrong",
            "password1": "new_password",
            "password2": "new_password"
        }
        response = self.client.post(url, new_password)
        self.assertContains(response, 'Current password', status_code=400)

        # POST 400: password mismatch
        new_password = {
            "current_password": "tester",
            "password1": "new_password",
            "password2": "wrong"
        }
        response = self.client.post(url, new_password)
        self.assertContains(response, 'mismatch', status_code=400)

        # POST 200: password changed
        new_password = {
            "current_password": "tester",
            "password1": "new_password",
            "password2": "new_password"
        }
        response = self.client.post(url, new_password)
        self.assertContains(response, 'successfully')

        # ensure password has really changed
        user = User.objects.get(username='registered')
        self.assertEqual(user.check_password('new_password'), True)

        self.client.logout()
        self.client.login(username='registered', password='new_password')

        # GET 405 again
        response = self.client.get(url)
        self.assertEqual(405, response.status_code)

    def test_account_password_change_empty_API(self):
        url = reverse('api_account_password_change')
        response = self.client.post(url, {
            "current_password": "",
            "password1": "",
            "password2": ""
        }, HTTP_ACCEPT='text/html')
        self.assertContains(response, 'current_password', status_code=400)

        response = self.client.post(url, {
            "current_password": "cc",
            "password1": "cc",
            "password2": "cc"
        }, HTTP_ACCEPT='text/html')
        self.assertContains(response, 'current_password', status_code=400)

        response = self.client.post(url, {
            "current_password": "tester",
            "password1": "changed",
            "password2": "changed"
        }, HTTP_ACCEPT='text/html')
        self.assertContains(response, 'changed', status_code=200)

    def test_account_password_reset_API(self):
        url = reverse('api_account_password_reset_request_key')

        # GET 403 - user must not be authenticated
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)

        self.client.logout()

        # GET 405
        response = self.client.get(url)
        self.assertEqual(405, response.status_code)

        # POST 400: missing required field
        response = self.client.post(url)
        self.assertContains(response, 'required', status_code=400)

        # POST 400: email not found in the DB
        response = self.client.post(url, {'email': 'imnotin@the.db'})
        self.assertContains(response, 'not found', status_code=400)

        # POST 200
        user = User.objects.get(username='registered')
        response = self.client.post(url, {'email': user.email})
        self.assertEqual(200, response.status_code)
        self.assertEqual(PasswordReset.objects.filter(user=user).count(), 1, 'dummy email outbox should contain 1 email message')
        self.assertEqual(len(mail.outbox), 1, 'dummy email outbox should contain 1 email message')

        password_reset = PasswordReset.objects.get(user=user)
        uid_36 = int_to_base36(user.id)
        url = reverse('api_account_password_reset_from_key',
                      kwargs={'uidb36': uid_36, 'key': password_reset.temp_key})

        # POST 400: wrong password
        params = {'password1': 'new_password', 'password2': 'wrong'}
        response = self.client.post(url, params)
        self.assertContains(response, '"password2"', status_code=400)

        # correct
        params['password2'] = 'new_password'
        response = self.client.post(url, json.dumps(params), content_type='application/json')
        self.assertContains(response, '"detail"')

        # ensure password has been changed
        user = User.objects.get(username='registered')
        self.assertTrue(user.check_password('new_password'))

        # ensure password reset object has been used
        password_reset = PasswordReset.objects.get(user=user)
        self.assertTrue(password_reset.reset)

        # request a new password reset key
        response = self.client.post(reverse('api_account_password_reset_request_key'), {'email': user.email})
        self.assertEqual(200, response.status_code)

        # test HTML version of password reset from key
        password_reset = PasswordReset.objects.get(user=user, reset=False)
        uid = int_to_base36(password_reset.user_id)
        key = password_reset.temp_key
        url = reverse('account_password_reset_from_key', args=[uid, key])

        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        response = self.client.post(url, {'password1': 'changed', 'password2': 'changed'})
        self.assertEqual(200, response.status_code)

        # ensure password has been changed
        user = User.objects.get(username='registered')
        self.assertTrue(user.check_password('changed'))

        # ensure password reset object has been used
        password_reset = PasswordReset.objects.filter(user=user).order_by('-id')[0]
        self.assertTrue(password_reset.reset)

    def test_account_password_reset_API_regression(self):
        url = reverse('api_account_password_reset_request_key')
        self.client.logout()
        user = User.objects.get(username='registered')
        response = self.client.post(url, {'email': user.email}, HTTP_ACCEPT='text/html')
        self.assertEqual(200, response.status_code)

    def test_account_login(self):
        url = reverse('api_account_login')

        # GET 403: already loggedin
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)

        self.client.logout()

        # POST 400 invalid credentials
        response = self.client.post(url, {"username": "wrong", "password": "wrong"})
        self.assertContains(response, 'Ivalid login credentials', status_code=400)

        # POST 400 missing credentials
        response = self.client.post(url)
        self.assertContains(response, 'required', status_code=400)

        # POST 200 login successfull
        response = self.client.post(url, {"username": "registered", "password": "tester"})
        self.assertContains(response, 'successful')

        # GET 403: already loggedin
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)

        self.client.logout()

        # POST 200 login successfull with email
        response = self.client.post(url, {"username": "registered@registered.org", "password": "tester"})
        self.assertContains(response, 'successful')

    def test_account_login_inactive(self):
        url = reverse('api_account_login')
        self.client.logout()

        # disable user
        user = User.objects.get(username='registered')
        user.is_active = False
        user.save()
        # expect 400
        response = self.client.post(url, {"username": "registered", "password": "tester"})
        self.assertContains(response, 'inactive', status_code=400)

        # re-enable user
        user.is_active = True
        user.save()
        # expect 200 login successfull
        response = self.client.post(url, {"username": "registered", "password": "tester"})
        self.assertContains(response, 'successful')

    def test_account_logout(self):
        url = reverse('api_account_logout')
        account_url = reverse('api_account_detail')

        # GET 405
        response = self.client.get(url)
        self.assertEqual(405, response.status_code)

        # Ensure is authenticated
        response = self.client.get(account_url)
        self.assertEqual(200, response.status_code)

        # POST 200
        response = self.client.post(url)
        self.assertEqual(200, response.status_code)

        # Ensure is NOT authenticated
        response = self.client.get(account_url)
        self.assertEqual(401, response.status_code)

    def test_social_links_api(self):
        url = reverse('api_user_social_links_list', args=['romano'])

        # GET 200
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        # POST 403 - only profile owner can submit new links to his profile
        response = self.client.post(url, {'url': 'http://mywebsite.com', 'description': 'mywebsite'})
        self.assertEqual(response.status_code, 403)

        # POST 201
        self.client.login(username='romano', password='tester')
        response = self.client.post(url, {'url': 'http://mywebsite.com', 'description': 'mywebsite'})
        self.assertEqual(response.status_code, 201)

        self.assertEqual(SocialLink.objects.count(), 1)

        url = reverse('api_user_social_links_detail', args=['romano', 1])

        # GET 200
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        data = json.dumps({'url': 'http://changed.com', 'description': 'changed'})

        # PUT 200
        response = self.client.put(url, data=data, content_type='application/json')
        self.assertEqual(response.status_code, 200)

        self.client.logout()
        self.client.login(username='registered', password='tester')

        # POST 403 - only profile owner can edit
        response = self.client.put(url, data=data, content_type='application/json')
        self.assertEqual(response.status_code, 403)

    if 'nodeshot.core.nodes' in settings.INSTALLED_APPS:
        def test_user_nodes(self):
            url = reverse('api_user_nodes', args=['romano'])

            response = self.client.get(url)
            self.assertEqual(200, response.status_code)

            response = self.client.post(url)
            self.assertEqual(405, response.status_code)

    if EMAIL_CONFIRMATION:
        def test_account_email_api(self):
            list_url = reverse('api_account_email_list')

            # GET 401 - unauthorized because not authenticated
            c = Client()
            response = c.get(list_url)
            self.assertEqual(401, response.status_code)

            # GET 200
            response = self.client.get(list_url)
            self.assertEqual(200, response.status_code)

            # POST 400 - wrong input data
            response = self.client.post(list_url)
            self.assertEqual(400, response.status_code)

            # POST 201
            user = User.objects.get(username='registered')
            email_addresses_count = EmailAddress.objects.filter(user=user).count()
            response = self.client.post(list_url, {'email': 'testing@test.com'})
            self.assertEqual(201, response.status_code)
            self.assertEqual(EmailAddress.objects.filter(user=user).count(), email_addresses_count + 1)
            # ensure is not verified nor primary
            email_address = EmailAddress.objects.filter(user=user).order_by('-id')[0]
            self.assertFalse(email_address.primary)
            self.assertFalse(email_address.verified)
            # ensure email has been sent
            self.assertEqual(len(mail.outbox), 1)
            # get email confirmation object
            email_confirmation = EmailConfirmation.objects.filter(email_address=email_address)[0]

            detail_url = reverse('api_account_email_detail', args=[email_address.pk])

            # GET detail 200
            response = self.client.get(detail_url)
            self.assertEqual(200, response.status_code)
            self.assertNotEqual(response.data['resend_confirmation'], False)

            # PUT 400 - can't make primary an unverified email address
            response = self.client.put(detail_url, data=json.dumps({'primary': True}), content_type='application/json')
            self.assertContains(response, _('Email address cannot be made primary if it is not verified first'), status_code=400)

            # POST resend confirmation
            resend_confirmation_url = reverse('api_account_email_resend_confirmation', args=[email_address.pk])
            response = self.client.post(resend_confirmation_url)
            self.assertEqual(200, response.status_code)
            # ensure email has been sent
            self.assertEqual(len(mail.outbox), 2)
            # get email confirmation object
            email_confirmation = EmailConfirmation.objects.filter(email_address=email_address).order_by('-id')[0]

            # verify email
            confirmation_url = reverse('emailconfirmation_confirm_email', args=[email_confirmation.key])
            response = self.client.get(confirmation_url)
            self.assertEqual(response.status_code, 302)

            # ensure verified and not primary (user already had a primary address)
            email_address = EmailAddress.objects.get(user=user, email='testing@test.com')
            self.assertFalse(email_address.primary)
            self.assertTrue(email_address.verified)

            # try to resend confirmation, should return error
            resend_confirmation_url = reverse('api_account_email_resend_confirmation', args=[email_address.pk])
            response = self.client.post(resend_confirmation_url)
            self.assertEqual(400, response.status_code)

            # DELETE 204 - delete the other email address
            detail_url = reverse('api_account_email_detail', args=[email_address.pk])
            response = self.client.delete(detail_url)
            self.assertEqual(response.status_code, 204)
            self.assertEqual(EmailAddress.objects.filter(user=user, email='testing@test.com').count(), 0)

            primary = email_address.user.email_set.get_primary()
            detail_url = reverse('api_account_email_detail', args=[primary.pk])
            # PUT 400 - can't unprimary
            response = self.client.put(detail_url, data=json.dumps({'primary': False}), content_type='application/json')
            self.assertContains(response, _('You must have at least one primary address.'), status_code=400)

            # DELETE 400 - can't delete because only 1 address
            response = self.client.delete(detail_url)
            self.assertEqual(response.status_code, 400)

            # add a new email address
            response = self.client.post(list_url, {'email': 'mynewemailaddress@test.com'})
            self.assertEqual(201, response.status_code)
            self.assertEqual(len(mail.outbox), 3)

            # verify new email
            email_address = EmailAddress.objects.get(user=user, email='mynewemailaddress@test.com')
            email_confirmation = EmailConfirmation.objects.filter(email_address=email_address).order_by('-id')[0]
            confirmation_url = reverse('emailconfirmation_confirm_email', args=[email_confirmation.key])
            response = self.client.get(confirmation_url)
            self.assertEqual(response.status_code, 302)

            # ensure is verified but not primary
            email_address = EmailAddress.objects.get(user=user, email='mynewemailaddress@test.com')
            self.assertFalse(email_address.primary)
            self.assertTrue(email_address.verified)

            detail_url = reverse('api_account_email_detail', args=[email_address.pk])

            # PUT 200 - make primary and ensure other one is not primary anymore
            response = self.client.put(detail_url, data=json.dumps({'primary': True}), content_type='application/json')
            self.assertEqual(response.status_code, 200)
            # ensure is primary
            email_address = EmailAddress.objects.get(user=user, email='mynewemailaddress@test.com')
            self.assertTrue(email_address.primary)
            # ensure previous one is primary
            email_address = EmailAddress.objects.get(user=user, email='registered@registered.org')
            self.assertFalse(email_address.primary)

    def test_password_confirmation_field_in_html(self):
        url = reverse('api_profile_list')
        response = self.client.get(url, HTTP_ACCEPT='text/html')

        self.assertContains(response, 'password:</label>')
        self.assertContains(response, 'password_confirmation:</label>')

    def test_delete_user(self):
        User.objects.all().delete()
        self.assertEqual(User.objects.count(), 0)

    def test_email_address_primary(self):
        user = User.objects.get(username='registered')
        self.assertEqual(user.email_set.filter(primary=True).count(), 1)

        email_address2 = EmailAddress(user=user, email='test@test.com', verified=True, primary=True)
        email_address2.full_clean()
        email_address2.save()
        self.assertEqual(user.email_set.count(), 2)
        self.assertEqual(user.email_set.filter(primary=True).count(), 1)
        self.assertEqual(user.email_set.filter(primary=True).first().email, 'test@test.com')
        self.assertEqual(User.objects.get(username='registered').email, 'test@test.com')

    def test_email_address_remove_primary(self):
        user = User.objects.get(username='registered')
        self.assertEqual(user.email_set.filter(primary=True).count(), 1)
        # setting primary as False raises validation error
        email_address = user.email_set.filter(primary=True).first()
        email_address.primary = False
        with self.assertRaises(ValidationError):
            email_address.full_clean()

    def test_add_user_creates_email(self):
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'email': 'test@test.org',
            'password': 'test'
        })
        user.save()
        self.assertEqual(EmailAddress.objects.filter(email=user.email, primary=True).count(), 1)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 1)
        user.delete()

        # ensure no email doesn't cause errors
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'password': 'test'
        })
        user.save()
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)

    def test_set_email_to_user(self):
        # ensure no email doesn't cause errors
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'password': 'test'
        })
        user.save()
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)
        # adding email later must work too
        user.email = 'test@test.org'
        user.save()
        self.assertEqual(EmailAddress.objects.filter(email=user.email, primary=True).count(), 1)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 1)

        user.email = 'anotherone@another.org'
        user.save()
        self.assertEqual(user.email_set.count(), 2)
        self.assertEqual(user.email_set.get_primary().email, user.email)

    def test_user_manager_creates_email(self):
        user = User.objects.create_user(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'email': 'test@test.org',
            'password': 'test'
        })
        self.assertEqual(EmailAddress.objects.filter(email=user.email, primary=True).count(), 1)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 1)
        user.delete()

        # ensure no email doesn't cause errors
        user = User.objects.create_user(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'password': 'test'
        })
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)

    def test_add_email_to_new_user(self):
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'password': 'test'
        })
        user.save()
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)
        email_address = EmailAddress(user=user, email='test@test.com')
        email_address.full_clean()
        email_address.save()
        self.assertEqual(user.email_set.filter(primary=True).count(), 1)
        self.assertEqual(user.email_set.count(), 1)

    def test_skip_sync_emailaddress(self):
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'email': 'test@test.org',
            'password': 'test'
        })
        user.save(sync_emailaddress=False)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)
        user.delete()

        # ensure no email doesn't cause errors
        user = User(**{
            'first_name': 'test',
            'last_name': 'test',
            'username': 'test',
            'password': 'test'
        })
        user.save(sync_emailaddress=False)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)

    def test_user_manager_skip_sync_emailaddress(self):
        user = User.objects.create_user(
            sync_emailaddress=False,
            **{
                'first_name': 'test',
                'last_name': 'test',
                'username': 'test',
                'email': 'test@test.org',
                'password': 'test'
            }
        )
        self.assertEqual(EmailAddress.objects.filter(email=user.email, primary=True).count(), 0)
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)
        user.delete()

        # ensure no email doesn't cause errors
        user = User.objects.create_user(
            sync_emailaddress=False,
            **{
                'first_name': 'test',
                'last_name': 'test',
                'username': 'test',
                'password': 'test'
            }
        )
        self.assertEqual(EmailAddress.objects.filter(email=user.email).count(), 0)

    def test_profile_manager_exception(self):
        with self.assertRaises(ValueError):
            User.objects.create_user(username=None, first_name='test',
                                     last_name='test', password='test')

    def test_create_superuser(self):
        User.objects.create_superuser('newadmin', '', 'newadmin')
        user = User.objects.get(username='newadmin')
        self.assertEqual(user.email_set.count(), 0)

    if EMAIL_CONFIRMATION:
        def test_email_uniqueness(self):
            self.assertEqual(EmailAddress.objects.filter(email='admin@admin.org').count(), 1)
            user = User.objects.get(username='romano')
            email = EmailAddress(user=user, email='admin@admin.org')
            with self.assertRaises(ValidationError):
                email.full_clean()



from django.core.management.base import BaseCommand

from nodeshot.core.websockets.server import start as start_server


class Command(BaseCommand):
    help = "Start Tornado WebSocket Server"

    def handle(self, *args, **options):
        """ Go baby go! """
        start_server()



links_legend = [
    {
        "name": "Link",
        "slug": "link",
        "description": "Active links",
        "fill_color": "#00ff00",
        "stroke_color": "#00ff00",
        "stroke_width": 4,
        "text_color": "#ffffff",
    }
]
